@@ -12,16 +12,16 @@
 
 /* Deprecation / Availability macros */
 
-#if !defined(NM_VERSION_MIN_REQUIRED) || (NM_VERSION_MIN_REQUIRED == 0)
-#undef NM_VERSION_MIN_REQUIRED
-#define NM_VERSION_MIN_REQUIRED (NM_API_VERSION)
-#endif
-
 #if !defined(NM_VERSION_MAX_ALLOWED) || (NM_VERSION_MAX_ALLOWED == 0)
 #undef NM_VERSION_MAX_ALLOWED
 #define NM_VERSION_MAX_ALLOWED (NM_API_VERSION)
 #endif
 
+#if !defined(NM_VERSION_MIN_REQUIRED) || (NM_VERSION_MIN_REQUIRED == 0)
+#undef NM_VERSION_MIN_REQUIRED
+#define NM_VERSION_MIN_REQUIRED (NM_VERSION_MAX_ALLOWED)
+#endif
+
 /* sanity checks */
 #if NM_VERSION_MIN_REQUIRED > NM_API_VERSION
 #error "NM_VERSION_MIN_REQUIRED must be <= NM_API_VERSION"
@@ -439,16 +439,18 @@
 #define NM_AVAILABLE_IN_1_54
 #endif
 
-#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_54_2
-#define NM_AVAILABLE_IN_1_54_2 G_UNAVAILABLE(1, 54.2)
+#if NM_VERSION_MIN_REQUIRED >= NM_VERSION_1_56
+#define NM_DEPRECATED_IN_1_56        G_DEPRECATED
+#define NM_DEPRECATED_IN_1_56_FOR(f) G_DEPRECATED_FOR(f)
 #else
-#define NM_AVAILABLE_IN_1_54_2
+#define NM_DEPRECATED_IN_1_56
+#define NM_DEPRECATED_IN_1_56_FOR(f)
 #endif
 
-#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_54_3
-#define NM_AVAILABLE_IN_1_54_3 G_UNAVAILABLE(1, 54.3)
+#if NM_VERSION_MAX_ALLOWED < NM_VERSION_1_56
+#define NM_AVAILABLE_IN_1_56 G_UNAVAILABLE(1, 56)
 #else
-#define NM_AVAILABLE_IN_1_54_3
+#define NM_AVAILABLE_IN_1_56
 #endif
 
 /*
